.app-header {
  background: rgba(20, 20, 20, 0.8) !important;
  backdrop-filter: blur(20px);
  border-bottom: 1px solid rgba(255, 255, 255, 0.08);
  padding: 0;
  position: sticky;
  top: 0;
  z-index: 1000;
}

.header-container {
  max-width: 1440px;
  margin: 0 auto;
  padding: 0 24px;
  display: flex;
  align-items: center;
  justify-content: space-between;
  height: 64px;
}

.header-left {
  display: flex;
  align-items: center;
}

.app-logo {
  display: flex;
  align-items: center;
  gap: 12px;
  color: white;
  font-size: 20px;
  font-weight: 700;
  cursor: pointer;
}

.logo-icon {
  font-size: 24px;
  color: var(--primary-color);
}

.logo-text {
  background: linear-gradient(45deg, #1890ff, #52c41a);
  -webkit-background-clip: text;
  -webkit-text-fill-color: transparent;
  background-clip: text;
}

.header-center {
  flex: 1;
  display: flex;
  justify-content: center;
}

.header-menu {
  background: transparent !important;
  border: none !important;
}

.header-menu .ant-menu-item {
  color: rgba(255, 255, 255, 0.65) !important;
}

.header-menu .ant-menu-item:hover {
  color: white !important;
}

.header-menu .ant-menu-item-selected {
  color: var(--primary-color) !important;
}

.header-right {
  display: flex;
  align-items: center;
}

.refresh-btn {
  color: rgba(255, 255, 255, 0.65) !important;
}

.refresh-btn:hover {
  color: white !important;
}